@@ -125,6 +125,8 @@ enum
 {
   PROP_CHANNEL = 1,
   PROP_FARSIGHT_CONFERENCE,
+  PROP_FARSIGHT_SESSION,
+  PROP_FARSIGHT_STREAM,
   PROP_FARSIGHT_PARTICIPANT,
   PROP_PROXY,
   PROP_STREAM_ID,
@@ -180,7 +182,7 @@ static void start_telephony_event (TpMediaStreamHandler *proxy, guchar event,
 static void stop_telephony_event (TpMediaStreamHandler *proxy,
     gpointer user_data, GObject *object);
 
-static void close (TpMediaStreamHandler *proxy,
+static void stream_close (TpMediaStreamHandler *proxy,
     gpointer user_data, GObject *object);
 
 static void invalidated_cb (TpMediaStreamHandler *proxy,
@@ -238,6 +240,12 @@ tf_stream_get_property (GObject    *object,
     case PROP_FARSIGHT_PARTICIPANT:
       g_value_set_object (value, self->priv->fs_participant);
       break;
+    case PROP_FARSIGHT_SESSION:
+      g_value_set_object (value, self->priv->fs_session);
+      break;
+    case PROP_FARSIGHT_STREAM:
+      g_value_set_object (value, self->priv->fs_stream);
+      break;
     case PROP_PROXY:
       g_value_set_object (value, self->priv->stream_handler_proxy);
       break;
@@ -465,6 +473,20 @@ tf_stream_class_init (TfStreamClass *klass)
           FS_TYPE_PARTICIPANT,
           G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  g_object_class_install_property (object_class, PROP_FARSIGHT_SESSION,
+      g_param_spec_object ("farsight-session",
+          "Farsight session",
+          "The Farsight session",
+          FS_TYPE_SESSION,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_property (object_class, PROP_FARSIGHT_STREAM,
+      g_param_spec_object ("farsight-stream",
+          "Farsight stream",
+          "The Farsight stream",
+          FS_TYPE_STREAM,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
   g_object_class_install_property (object_class, PROP_PROXY,
       g_param_spec_object ("proxy",
           "TpMediaStreamHandler proxy",
@@ -668,6 +690,7 @@ get_all_properties_cb (TpProxy *proxy,
   gboolean created_locally = TRUE;
   gboolean valid = FALSE;
   guint i;
+  gboolean do_controlling = FALSE;
 
   if (dbus_error &&
       !(dbus_error->domain == DBUS_GERROR &&
@@ -709,7 +732,7 @@ get_all_properties_cb (TpProxy *proxy,
       (stream->priv->stream_handler_proxy, stop_telephony_event, NULL, NULL,
           (GObject*) stream, NULL);
   tp_cli_media_stream_handler_connect_to_close
-      (stream->priv->stream_handler_proxy, close, NULL, NULL,
+      (stream->priv->stream_handler_proxy, stream_close, NULL, NULL,
           (GObject*) stream, NULL);
 
   memset (params, 0, sizeof(GParameter) * MAX_STREAM_TRANS_PARAMS);
@@ -722,6 +745,7 @@ get_all_properties_cb (TpProxy *proxy,
   if (!nat_traversal || !strcmp (nat_traversal, "gtalk-p2p"))
     {
       transmitter = "nice";
+      do_controlling = TRUE;
 
       params[n_args].name = "compatibility-mode";
       g_value_init (&params[n_args].value, G_TYPE_UINT);
@@ -731,10 +755,12 @@ get_all_properties_cb (TpProxy *proxy,
   else if (!strcmp (nat_traversal, "ice-udp"))
     {
       transmitter = "nice";
+      do_controlling = TRUE;
     }
   else if (!strcmp (nat_traversal, "wlm-8.5"))
     {
       transmitter = "nice";
+      do_controlling = TRUE;
 
       params[n_args].name = "compatibility-mode";
       g_value_init (&params[n_args].value, G_TYPE_UINT);
@@ -744,12 +770,17 @@ get_all_properties_cb (TpProxy *proxy,
   else if (!strcmp (nat_traversal, "wlm-2009"))
     {
       transmitter = "nice";
+      do_controlling = TRUE;
 
       params[n_args].name = "compatibility-mode";
       g_value_init (&params[n_args].value, G_TYPE_UINT);
       g_value_set_uint (&params[n_args].value, 3);
       n_args++;
     }
+  else if (!strcmp (nat_traversal, "shm"))
+    {
+      transmitter = "shm";
+    }
   else
     {
       transmitter = "rawudp";
@@ -841,7 +872,6 @@ get_all_properties_cb (TpProxy *proxy,
   if (dbus_relay_info && dbus_relay_info->len)
     {
       GValueArray *fs_relay_info = g_value_array_new (0);
-      guint i;
       GValue val = {0};
       g_value_init (&val, GST_TYPE_STRUCTURE);
 
@@ -902,7 +932,7 @@ get_all_properties_cb (TpProxy *proxy,
       g_value_array_free (fs_relay_info);
     }
 
-  if (out_Properties)
+  if (out_Properties && do_controlling)
     {
       created_locally = tp_asv_get_boolean (out_Properties, "CreatedLocally",
           &valid);
@@ -1752,7 +1782,7 @@ tf_stream_shutdown (TfStream *self)
 
 
 static void
-close (TpMediaStreamHandler *proxy G_GNUC_UNUSED,
+stream_close (TpMediaStreamHandler *proxy G_GNUC_UNUSED,
        gpointer user_data G_GNUC_UNUSED,
        GObject *object)
 {
@@ -1878,7 +1908,6 @@ _tf_stream_bus_message (TfStream *stream,
 {
   const gchar *debug = NULL;
   const GstStructure *s = gst_message_get_structure (message);
-  const GValue *value = NULL;
 
   if (GST_MESSAGE_TYPE (message) != GST_MESSAGE_ELEMENT)
     return FALSE;
@@ -1889,6 +1918,7 @@ _tf_stream_bus_message (TfStream *stream,
   if (gst_structure_has_name (s, "farsight-error"))
     {
       GObject *object;
+      const GValue *value = NULL;
 
       value = gst_structure_get_value (s, "src-object");
       object = g_value_get_object (value);
